package ru.dbotthepony.mc.otm.menu

import net.minecraft.core.Direction
import kotlin.jvm.JvmOverloads
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.level.block.Block
import ru.dbotthepony.mc.otm.block.BlockEnergyCounter
import ru.dbotthepony.mc.otm.block.entity.BlockEntityEnergyCounter
import ru.dbotthepony.mc.otm.menu.data.EnumDataContainer
import ru.dbotthepony.mc.otm.menu.data.ImpreciseFractionDataContainer
import ru.dbotthepony.mc.otm.menu.widget.OneWayPlayerInputWidget
import ru.dbotthepony.mc.otm.registry.MMenus

class MenuEnergyCounter @JvmOverloads constructor(
	p_38852_: Int,
	inventory: Inventory,
	tile: BlockEntityEnergyCounter? = null
) : MatteryMenu(MMenus.ENERGY_COUNTER, p_38852_, inventory, tile) {
	val passed = ImpreciseFractionDataContainer()
	val average = ImpreciseFractionDataContainer()
	val last20Ticks = ImpreciseFractionDataContainer()
	val lastTick = ImpreciseFractionDataContainer()

	val switchDirection: OneWayPlayerInputWidget
	val inputDirection = EnumDataContainer(Direction::class.java)

	// TODO: Graph and proper networking for it
	private var ticksPassed = 0

	init {
		if (tile == null) {
			switchDirection = OneWayPlayerInputWidget(this)
		} else {
			switchDirection = OneWayPlayerInputWidget(this) {
				tile.level?.setBlock(tile.blockPos, tile.blockState.setValue(BlockEnergyCounter.INPUT_DIRECTION, tile.blockState.getValue(BlockEnergyCounter.INPUT_DIRECTION).opposite), Block.UPDATE_ALL)
			}
		}

		addDataSlots(passed)
		addDataSlots(average)
		addDataSlots(last20Ticks)
		addDataSlots(lastTick)
		addDataSlots(inputDirection)
	}

	override fun broadcastChanges() {
		if (tile is BlockEntityEnergyCounter) {
			passed.value = tile.passed
			average.value = tile.calcAverage(20)
			lastTick.value = tile.lastTick

			if (ticksPassed == 0) {
				last20Ticks.value = tile.sumHistory(20)
			}

			ticksPassed = (ticksPassed + 1) % 20
			inputDirection.value = tile.blockState.getValue(BlockEnergyCounter.INPUT_DIRECTION)
		}

		super.broadcastChanges()
	}

	override fun getWorkingSlotStart(): Int {
		return 0
	}

	override fun getWorkingSlotEnd(): Int {
		return 0
	}
}